/*
 * Copyright (C) 2013-2015 by Adaptive Computing Enterprises, Inc.
 * All Rights Reserved.
 *
 * Upgrades the schema of the mws database.
 */

function contains(array, obj) {
	for (var i = 0; i < array.length; i++) {
		if (array[i] == obj)
			return true;
	}
	return false;
}

function isChangeSetApplied(changeSet) {
	return contains(db.getCollectionNames(), "mongeez") && db.mongeez.count({changeId: changeSet}) > 0;
}

function applyChangeSet(changeSet, file) {
	db.mongeez.insert({
		type: "changeSetExecution",
		file: file,
		changeId: changeSet,
		author: "mws",
		resourcePath: null,
		date: new Date()
	})
}

function main() {
	db = db.getSiblingDB("mws");

	// 7.2 migrations
	if (!isChangeSetApplied('7.2 hypervisorType migration')) {
		db.image.find({"hypervisor": true, "hypervisorType": {$in: [undefined, null]}, "extensions.xcat.hvType": {$in: [undefined, null]}}).forEach(function (image) {
			image.hypervisorType = "ESX";
			db.image.save(image);
		});

		db.image.find({"extensions.xcat.hvType": {$ne: null}}).forEach(function (image) {
			if (image.hypervisorType == null || image.hypervisorType == undefined) {
				var hvType = image.extensions.xcat.hvType;
				if (hvType == undefined || hvType == null || !hvType.match(/^[a-zA-Z0-9-._]+$/))
					hvType = "ESX";
				image.hypervisorType = hvType.toUpperCase();
			}
			delete image.extensions.xcat.hvType;
			db.image.save(image);
		});

		applyChangeSet('7.2 hypervisorType migration', '7.2-migrations.xml')
	}

	// 7.5 migrations for events, services, and service templates
	if (!isChangeSetApplied('7.5 migrations')) {
		var severityCodes = {
			INFO: 0x0,
			WARN: 0x1,
			ERROR: 0x2,
			FATAL: 0x3
		};
		db.event.find().forEach(function (event) {
			// Translate fields to new format
			event.eventDate = event.eventTime;
			delete event.eventTime;
			event.origin = event.sourceComponent;
			delete event.sourceComponent;
			if (event.errorMessage)
				event.message = event.errorMessage.message;
			delete event.errorMessage;
			event.associatedObjects = [];
			if (event.primaryObject && event.primaryObject._id)
				event.associatedObjects.push({_id: event.primaryObject._id, type: event.primaryObject.type});
			delete event.primaryObject;
			if (event.relatedObjects) {
				event.relatedObjects.forEach(function (obj) {
					event.associatedObjects.push({_id: obj._id, type: obj.type});
				});
			}
			delete event.relatedObjects;
			event.code = 0x0DFFFFFF;	// Unknown code

			event.arguments = [];

			// Remove deprecated fields
			delete event.details;
			delete event.eventCategory;
			delete event.status;
			delete event.facility;
			delete event.initiatedBy;

			// Set severity appropriately - default to info
			if (event.severity != null && severityCodes[event.severity] != undefined && severityCodes[event.severity] != null)
				event.code = event.code | (severityCodes[event.severity] << 28);
			else
				event.severity = "INFO";

			// Save translated event
			db.event.save(event);
		});

		db.serviceTemplate.find({$or: [
			{"label": null},
			{"label": ""}
		]}).forEach(function (serviceTemplate) {
				serviceTemplate.label = serviceTemplate.name;
				db.serviceTemplate.save(serviceTemplate);
			});

		db.service.find({$or: [
			{"label": null},
			{"label": ""}
		]}).forEach(function (service) {
				service.label = service.name;
				db.service.save(service);
			});

		applyChangeSet('7.5 migrations', '7.5-migrations.xml')
	}

	// 7.5 RBAC migrations
	if (!isChangeSetApplied('7.5 RBAC 1 migrations')) {
		var placeholderTenantName = "Organization";
		switch (db.tenant.count()) {
			case 0:
				var now = new Date();
				db.tenant.insert({
					_id: ObjectId().str,
					name: placeholderTenantName,
					description: "Created during MWS 7.5 upgrade on " +
						now.getFullYear() + "-" +
						(now.getMonth() + 1) + "-" +
						now.getDate(),
					attachedNodes: [],
					attachedPrincipals: [],
					version: 0
				});
				break;
			case 1:
				// A tenant exists. We check for the correct tenant below.
				break;
			default:
				print("The database upgrade cannot complete automatically. " +
					"More than one tenant exists. " +
					"Please contact customer support for help.");
				return;
		}
		var tenant = db.tenant.findOne({name: placeholderTenantName});
		if (!tenant) {
			print("The database upgrade cannot complete automatically. " +
				"Cannot find or create the placeholder tenant. " +
				"Please contact customer support for help.");
			return;
		}

		var tenantProperty = { name: tenant.name, id: tenant._id };
		db.service.find().forEach(function (obj) {
			obj.tenant = tenantProperty;
			db.service.save(obj)
		});
		db.serviceTemplate.find().forEach(function (obj) {
			obj.tenant = tenantProperty;
			db.serviceTemplate.save(obj)
		});
		db.event.find().forEach(function (obj) {
			obj.tenant = tenantProperty;
			db.event.save(obj)
		});

		db.permission.find().forEach(function (permission) {
			if (permission.type != 'domain') {
				permission.scope = 'NONE';
				permission.fieldPath = null
			}
			db.permission.save(permission)
		});
		db.role.find().forEach(function (role) {
			if (role.name == 'PowerUser' || role.name == 'User') {
				role.scope = 'TENANT'
			}
			else {
				role.scope = 'GLOBAL'
			}
			db.role.save(role)
		});

		applyChangeSet('7.5 RBAC 1 migrations', '7.5-rbac-1-migrations.xml')
	}
}

main();
